#!/usr/bin/python
#------------------------------------------------------------------------------
# Copyright (c) 2007-2014 Perforce Software, Inc.  Provided for use as defined in
# the Perforce Consulting Services Agreement.
#------------------------------------------------------------------------------

"""
Usage:
	python checkusers_not_in_group.py

This script will generate a list of all standard Perforce user names that are not listed in any group.
It prints the results to the screen, so you may want to redirect it to a file.
"""

import os
import re
import string
import sys
import platform
import ConfigParser

if len(sys.argv) > 1:
  P4INSTANCE = str(sys.argv[1])
else:
  P4INSTANCE = '1'

config = ConfigParser.RawConfigParser()
config.read('maintenance.cfg')

##########################################################################
#####                                                                #####
#####  CONFIGURATION VARIABLES: Modify in maintenance.cfg as needed. #####
#####                                                                #####

server = (config.get(P4INSTANCE, 'server'))
p4user = (config.get(P4INSTANCE, 'p4user'))
passwd = (config.get(P4INSTANCE, 'passwd'))
weeks = int(config.get(P4INSTANCE, 'weeks'))

if platform.system() == "Windows": 
  p4="p4.exe -p %s -u %s" % (server, p4user)
else:
  p4="/p4/1/bin/p4_1  -p %s -u %s" % (server, p4user)

os.system('echo %s| %s login' % (passwd, p4))

users = []
groupusers = []
groups = []

try:
    for user in os.popen( "%s users" % p4 ).readlines():
        user = re.sub( r" <.*", r"", user )
        user = user.strip()
        user = user.lower()
        users.append( user )
except:
    print("Non unicode user name on server. Some user names may have been skipped.")

for group in os.popen( "%s groups" % p4).readlines():
    group = group.strip()
    groups.append( group )

for group in groups:
    pastUsers = 0
    for user in os.popen( "%s group -o %s" % (p4, group)).readlines():
        user = user.strip()
        if re.match( r"Users:", user ):
            pastUsers = 1
            continue
        if pastUsers == 1:
            user = user.lower()
            if user in groupusers:
                continue
            else:
                groupusers.append( user )

groupusers.sort()

for user in users:
    if user in groupusers:
        continue
    else:
        print(user)
